package org.example.wedding_app_backend.wedding.controller;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.dto.GetWeddingResponse;
import org.example.wedding_app_backend.wedding.function.WeddingMapper;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.UUID;

@RequestMapping("/api/wedding")
@RestController
@RequiredArgsConstructor
public class WeddingController {

    private final WeddingService weddingService;

    private final WeddingMapper weddingMapper;

    @GetMapping("/{id}")
    public ResponseEntity<GetWeddingResponse> getWedding(@PathVariable UUID id) {
        return weddingService.findWedding(id)
                .map(weddingMapper::entityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping(path = "{weddingId}/background", produces = MediaType.IMAGE_JPEG_VALUE)
    public ResponseEntity<byte[]> getWeddingImage(@PathVariable("weddingId") UUID weddingId) {
        return weddingService.findWedding(weddingId)
                .map(WeddingEntity::getBackgroundImage)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());

    }

    @PostMapping(path = "{weddingId}/background", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    public void putWeddingImage(@PathVariable("weddingId") UUID weddingId, @RequestParam("image") MultipartFile request) {
//        weddingService.save(requestToWeddingBackgroundImage.apply(weddingId, request));
    }
}
