package org.example.wedding_app_backend.weddingGuest.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.weddingGuest.dto.*;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.function.WeddingGuestMapper;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

@RequestMapping("/api/wedding")
@RestController
@Log
@RequiredArgsConstructor
public class WeddingGuestController {

    private final WeddingGuestService weddingGuestService;

    private final WeddingGuestMapper weddingGuestMapper;

    @GetMapping("{weddingId}/guest/{guestId}")
    public ResponseEntity<GetWeddingGuestResponse> getWeddingGuest(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId, guestId)
                .map(weddingGuestMapper::entityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("{weddingId}/guest")
    public ResponseEntity<List<GetWeddingGuestResponse>> getWeddingGuests(@PathVariable("weddingId") UUID weddingId) {
        return weddingGuestService.findAllByWeddingId(weddingId)
                .map(weddingGuestMapper::entitiesToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("{weddingId}/{guestId}/guests")
    public ResponseEntity<List<GetWeddingGuestInformation>> getWeddingGuestsData(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findAllByWeddingId(weddingId)
                .map(weddingGuestMapper::guestsDataEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("{weddingId}/{guestId}/quiz/state")
    public ResponseEntity<GetWeddingGuestQuizState> getWeddingGuestState(@PathVariable("weddingId") UUID weddingId,@PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::quizEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("{weddingId}/{guestId}/quiz/score")
    public ResponseEntity<GetWeddingGuestQuizScore> getWeddingGuestScore(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::quizScoreEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("{weddingId}/{guestId}/games/hall")
    public ResponseEntity<List<GetWeddingGuestsFinalScores>> getWeddingQuizScores(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findAllWeddingGuestsThatFinishedQuiz(weddingId)
                .map(weddingGuestMapper::quizScoresEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("/{weddingId}/{guestId}/games/who-knows-better/state")
    public ResponseEntity<GetWeddingGuestWhoKnowsBetterQuizState> getWhoKnowsBetterQuizState(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::whoKnowsBetterQuizEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @PatchMapping(path = "{weddingId}/{guestId}/guests")
    public void updateWeddingGuest(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody Map<String, String> payload)
    {
        {
            Optional<WeddingGuestEntity> guest = weddingGuestService.findWeddingGuest(UUID.fromString(payload.get("id")));

            if (guest.isPresent()) {
                weddingGuestService.updateGuest(payload);
            } else {
                throw new ResponseStatusException(HttpStatus.NOT_FOUND);
            }
        }
    }

    @PostMapping(path = "{weddingId}/{guestId}/guests")
    public void createWeddingGuest(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody Map<String, String> payload)
    {
        UUID guestPayloadId = UUID.fromString(String.valueOf(payload.get("id")));
        Optional<WeddingGuestEntity> guest = weddingGuestService.findWeddingGuest(guestPayloadId);

        if (guest.isEmpty()) {
            weddingGuestService.createGuest(payload,weddingId);
        } else {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }
    }

    @DeleteMapping(path = "{weddingId}/{guestId}/guests")
    public void deleteWeddingGuest(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody Map<String, String> payload)
    {
        Optional<WeddingGuestEntity> guest = weddingGuestService.findWeddingGuest(UUID.fromString(payload.get("id")));

        if (guest.isPresent()) {
            weddingGuestService.deleteGuest(payload);
        } else {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }
    }

}
