package org.example.wedding_app_backend.weddingGuest.service;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.weddingQuiz.entity.WeddingQuizEntity;
import org.example.wedding_app_backend.weddingQuiz.repository.WeddingQuizRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@RequiredArgsConstructor
public class WeddingGuestService {

    private final WeddingGuestRepository weddingGuestRepository;

    private final WeddingRepository weddingRepository;

    private final WeddingQuizRepository weddingQuizRepository;

    private final WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    public void save(WeddingGuestEntity weddingGuest) {
        weddingGuestRepository.save(weddingGuest);
    }

    public void delete(WeddingGuestEntity weddingGuest) {weddingGuestRepository.delete(weddingGuest);}

    public Optional<WeddingGuestEntity> findWeddingGuestById(UUID weddingId, UUID guestId) {
        return weddingGuestRepository.findByWeddingIdAndId(weddingId, guestId);
    }

    public Optional<List<WeddingGuestEntity>> findAllByWeddingId(UUID weddingId) {
        return weddingGuestRepository.findAllByWeddingId(weddingId);
    }

    public Optional<WeddingGuestEntity> findWeddingGuest(UUID id) {
        return weddingGuestRepository.findById(id);
    }

    public Optional<List<WeddingGuestEntity>> findAllWeddingGuestsThatFinishedQuiz(UUID id){return weddingGuestRepository.findAllByWeddingIdAndQuizFinishedTrue(id);}

    public void updateQuizPoints(UUID WeddingGuestId, int score) {
        weddingGuestRepository.findById(WeddingGuestId).ifPresent(weddingGuest -> {
            weddingGuest.setQuizPoints(weddingGuest.getQuizPoints() + score);
            weddingGuestRepository.save(weddingGuest);
        });
    }

    public void updateQuizIndex(UUID WeddingGuestId, UUID WeddingId) {
        WeddingGuestEntity guest = weddingGuestRepository.findById(WeddingGuestId)
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " + WeddingGuestId + " not found."));

        guest.setQuestionIndex(guest.getQuestionIndex() + 1);

        Optional<WeddingQuizEntity> question = weddingQuizRepository.findQuestionById(guest.getQuestionIndex(),WeddingId,guest.getQuizSeed());

        if(question.isEmpty()){
            guest.setQuizFinished(true);
        }
        weddingGuestRepository.save(guest);
    }

    public void updateWhoKnowsBetterQuizIndex(UUID WeddingGuestId, UUID WeddingId) {

        WeddingGuestEntity guest = weddingGuestRepository.findById(WeddingGuestId)
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " + WeddingGuestId + " not found."));

        guest.setWhoKnowsQuestionIndex(guest.getWhoKnowsQuestionIndex() + 1);

        Optional<WhoKnowsBetterQuiz> nextQuestion = whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(WeddingId, guest.getWhoKnowsQuestionIndex());

        if(nextQuestion.isEmpty()){
            guest.setWhoKnowsQuizFinished(true);
        }

        weddingGuestRepository.save(guest);
    }

    public void updateGuest(Map<String, String> payload) {
        WeddingGuestEntity guest = weddingGuestRepository.findById(UUID.fromString(payload.get("id")))
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " +UUID.fromString(payload.get("id"))+ " not found."));

        guest.setGuestName(payload.get("guestName"));
        guest.setGuestSurname(payload.get("guestSurname"));
        guest.setGuestAge(payload.get("guestAge"));

        weddingGuestRepository.save(guest);
    }

    public void createGuest(Map<String, String> payload,UUID weddingId) {
        WeddingGuestEntity guest = new WeddingGuestEntity();

        guest.setId(UUID.fromString(payload.get("id")));
        guest.setQuestionIndex(0);
        guest.setQuizPoints(0);
        guest.setQuizFinished(false);
        guest.setQuizSeed((int) (Math.random()*3)+3);
        guest.setGuestName(payload.get("guestName"));
        guest.setGuestSurname(payload.get("guestSurname"));
        guest.setGuestAge(payload.get("guestAge"));
        guest.setWhoKnowsQuestionIndex(0);
        guest.setWhoKnowsQuizFinished(false);

        WeddingEntity wedding = weddingRepository.findById(weddingId)
                .orElseThrow(() -> new NoSuchElementException("Wedding with ID " +weddingId+ " not found."));
        guest.setWedding(wedding);

        weddingGuestRepository.save(guest);
    }

    public void deleteGuest(Map<String, String> payload) {
        WeddingGuestEntity guest = weddingGuestRepository.findById(UUID.fromString(payload.get("id")))
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " +UUID.fromString(payload.get("id"))+ " not found."));

        weddingGuestRepository.delete(guest);
    }

}
