package org.example.wedding_app_backend.whoKnowsBetterQuiz.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.dto.GetWhoKnowsBetterQuizResponse;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.dto.UpdateAnswerRequest;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.function.WhoKnowsBetterQuizMapper;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.service.WhoKnowsBetterQuizService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.messaging.support.AbstractSubscribableChannel;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;
@RequestMapping("/api/wedding/{weddingId}/{guestId}/games")
@RestController
@Log
@RequiredArgsConstructor
public class WhoKnowsBetterQuizController {

    private final WhoKnowsBetterQuizService whoKnowsBetterQuizService;

    private final WhoKnowsBetterQuizMapper whoKnowsBetterQuizMapper;

    private final WeddingGuestService weddingGuestService;

    private final SimpMessagingTemplate messagingTemplate;
    private final AbstractSubscribableChannel clientOutboundChannel;


    @GetMapping("/who-knows-better")
    public ResponseEntity<GetWhoKnowsBetterQuizResponse> getWhoKnowsBetterQuestion(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId) {
        return whoKnowsBetterQuizService.findQuestionByGuestId(weddingId, guestId)
                .map(whoKnowsBetterQuizMapper::entityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }


    @PatchMapping("/who-knows-better/answer")
    public ResponseEntity<Void> updateWhoKnowsBetterQuestionCounter(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody UpdateAnswerRequest answer) {

        Optional<WeddingGuestEntity> guest = weddingGuestService.findWeddingGuest(guestId);
        if (guest.isPresent()) {

            WhoKnowsBetterQuiz updatedQuiz = whoKnowsBetterQuizService.updateQuestionCounter(weddingId, guestId, answer.getAnswer());

            weddingGuestService.updateWhoKnowsBetterQuizIndex(guestId, weddingId);

            GetWhoKnowsBetterQuizResponse response = whoKnowsBetterQuizMapper.entityToRepresentation(updatedQuiz);

            messagingTemplate.convertAndSend("/topic/who-knows-better/results", response);

            return ResponseEntity.ok().build();
        } else {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }
    }





}
