import {AfterViewInit, Component, inject, OnInit, Input, signal, WritableSignal} from '@angular/core';
import {WeddingService} from '../../services/wedding.service';
import {WeddingType} from '../../models/wedding.type';
import {ActivatedRoute} from '@angular/router';
import {SliderModule} from 'primeng/slider';
import {Button} from 'primeng/button';

@Component({
  selector: 'app-wedding',
  standalone: true,
  imports: [
    SliderModule,
    Button
  ],
  templateUrl: './wedding.component.html',
  styleUrl: './wedding.component.css'
})
export class WeddingComponent implements OnInit, AfterViewInit {

  @Input()
  weddingId!: string;

  wedding: WeddingType | null = null;

  constructor(
    private weddingService : WeddingService
  ) {}

  ngOnInit(): void {
    this.weddingService.getWeddingById(this.weddingId).subscribe({
      next: (wedding) => this.wedding = wedding,
      error: (error) => console.error('Error getting wedding:', error)
    })
  }

  currentCard: number = 1;

  carrousel!: HTMLElement;

  backgroundImg!: HTMLElement;

  imgWidth!: number;

  ngAfterViewInit(): void {
    if (typeof document !== 'undefined') {
      this.carrousel = document.querySelector('.carrousel') as HTMLElement;
      this.backgroundImg = document.querySelector('#background-img') as HTMLElement;

      this.imgWidth = this.backgroundImg.clientWidth;
    }
  }

  moveBackgroundImg(amount: number): void {
    this.backgroundImg.animate(
      [
        { transform: `translateX(${this.backgroundImg.getAttribute('translateX')})` },
        { transform: `translateX(${amount}px)` }
      ],
      {
        duration: 5000,
        easing: 'ease-in-out',
        fill: 'forwards'
      }
    );
  }

  calculateBackgroundScrollAmount(): number{
    if (!isNaN(this.imgWidth)) {
      this.imgWidth = this.backgroundImg.clientWidth;
      return (this.imgWidth - document.documentElement.clientWidth) / 3;
    } else {
      console.error('Invalid image width');
      return 0;
    }
  }

  arrowClick(direction: 'left' | 'right'): void {
    if (direction === 'left' && this.currentCard > 0) {
      this.carrousel.scrollBy({ left: -10, behavior: 'smooth' });
      this.currentCard--;
    } else if (direction === 'right' && this.currentCard < 3) { // TODO: replace 3 with constant that is equal to cards number
      this.carrousel.scrollBy({ left: 10, behavior: 'smooth' });
      this.currentCard++;
    }
  }

  onCarrouselScroll(element: HTMLElement): void {
    this.currentCard = Math.round(element.scrollLeft / element.clientWidth);
    this.moveBackgroundImg(this.currentCard * this.calculateBackgroundScrollAmount() * -1)
  }

}
