import { Injectable, Inject, PLATFORM_ID } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';
import Keycloak from 'keycloak-js';
import { UserProfile } from './user-profile';

@Injectable({
  providedIn: 'root'
})
export class KeycloakService {
  private _keycloak: Keycloak | undefined;
  private _profile: UserProfile | undefined;
  private isBrowser: boolean;

  constructor(@Inject(PLATFORM_ID) private platformId: object) {
    this.isBrowser = isPlatformBrowser(this.platformId);
  }

  get keycloak() {
    if (!this._keycloak && this.isBrowser) {
      this._keycloak = new Keycloak({
        url: 'http://localhost:9090',
        realm: 'wedding-app-realm',
        clientId: 'bsn'
      });
    }
    return this._keycloak;
  }

  get profile(): UserProfile | undefined {
    return this._profile;
  }

  async init() {
    if (!this.isBrowser) {
      return;
    }

    const authenticated = await this.keycloak!.init({
      onLoad: 'login-required',
    });

    if (authenticated) {
      this._profile = (await this.keycloak!.loadUserProfile()) as UserProfile;
      this._profile.token = this.keycloak!.token || '';
    }
  }

  login() {
    return this.isBrowser ? this.keycloak!.login() : Promise.resolve();
  }

  logout() {
    return this.isBrowser ? this.keycloak!.logout({ redirectUri: 'http://localhost:4200' }) : Promise.resolve();
  }

}
